@@ -545,13 +545,9 @@ class S7CPU(object):
 
 	def reallocate(self, force=False):
 		if force or (self.specs.nrAccus == 4) != self.is4accu:
-			self.accu1, self.accu2 = Accu(), Accu()
-			if self.specs.nrAccus == 2:
-				self.accu3, self.accu4 = None, None
-			elif self.specs.nrAccus == 4:
-				self.accu3, self.accu4 = Accu(), Accu()
-			else:
-				assert(0)
+			self.accu1, self.accu2, self.accu3, self.accu4 =\
+				Accu(), Accu(), Accu(), Accu()
+			self.is4accu = (self.specs.nrAccus == 4)
 		if force or self.specs.nrTimers != len(self.timers):
 			self.timers = [ Timer(self, i)
 					for i in range(self.specs.nrTimers) ]
@@ -590,6 +586,7 @@ class S7CPU(object):
 			# System SFBs
 		}
 		self.symbolTable = SymbolTable()
+		self.is4accu = False
 		self.reallocate(force=True)
 		self.ar1 = Adressregister()
 		self.ar2 = Adressregister()
@@ -647,10 +644,6 @@ class S7CPU(object):
 		if self.cbScreenUpdate:
 			self.cbScreenUpdate(self.cbScreenUpdateData)
 
-	@property
-	def is4accu(self):
-		return self.accu4 is not None
-
 	def __runOB(self, block):
 		# Update timekeeping
 		self.updateTimestamp()
