@@ -27,12 +27,15 @@ from awlsim.core.enumeration import *
 
 class AwlSimError(Exception):
 	def __init__(self, message, cpu=None,
-		     rawInsn=None, insn=None, lineNr=None):
+		     rawInsn=None, insn=None, lineNr=None,
+		     sourceId=None, sourceName=None):
 		Exception.__init__(self, message)
 		self.cpu = cpu
 		self.rawInsn = rawInsn
 		self.insn = insn
 		self.lineNr = lineNr
+		self.sourceId = sourceId
+		self.sourceName = sourceName
 
 	def setCpu(self, cpu):
 		self.cpu = cpu
@@ -52,6 +55,18 @@ class AwlSimError(Exception):
 	def getInsn(self):
 		return self.insn
 
+	def setSourceId(self, sourceId):
+		self.sourceId = sourceId
+
+	def getSourceId(self):
+		return self.sourceId
+
+	def setSourceName(self, sourceName):
+		self.sourceName = sourceName
+
+	def getSourceName(self):
+		return self.sourceName
+
 	def setLineNr(self, lineNr):
 		self.lineNr = lineNr
 
@@ -88,8 +103,14 @@ class AwlSimError(Exception):
 		return errorStr
 
 	def doGetReport(self, title):
+		sourceName = self.getSourceName()
+		if sourceName:
+			sourceName = "source '%s' " % sourceName
+		else:
+			sourceName = ""
 		ret = [ "-- %s --\n" % title ]
-		ret.append("ERROR at line %s:\n" % self.getLineNrStr())
+		ret.append("ERROR at %sline %s:\n" %\
+			   (sourceName, self.getLineNrStr()))
 		ret.append("  \n%s\n" % str(self))
 		cpu = self.getCpu()
 		if cpu:
