@@ -2,7 +2,7 @@
 #
 # AWL parser
 #
-# Copyright 2012-2013 Michael Buesch <m@bues.ch>
+# Copyright 2012-2014 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,11 +27,13 @@ import re
 
 from awlsim.core.util import *
 from awlsim.core.datatypes import *
+from awlsim.core.project import *
 
 
 class RawAwlInsn(object):
 	def __init__(self, block):
 		self.block = block
+		self.sourceId = None
 		self.lineNr = 0
 		self.label = None
 		self.name = None
@@ -53,6 +55,12 @@ class RawAwlInsn(object):
 		ret.extend(self.getOperators())
 		return " ".join(ret)
 
+	def setSourceId(self, sourceId):
+		self.sourceId = sourceId
+
+	def getSourceId(self):
+		return self.sourceId
+
 	def setLineNr(self, newLineNr):
 		self.lineNr = newLineNr
 
@@ -265,6 +273,9 @@ class AwlParseTree(object):
 
 		self.curBlock = None
 
+		self.sourceId = None
+		self.sourceName = None
+
 class AwlParser(object):
 	EnumGen.start
 	STATE_GLOBAL			= EnumGen.item
@@ -292,6 +303,8 @@ class AwlParser(object):
 	STATE_IN_OB			= EnumGen.item
 	EnumGen.end
 
+	TEXT_ENCODING = "latin_1"
+
 	class TokenizerState(object):
 		def __init__(self, parser):
 			self.parser = parser
@@ -315,7 +328,7 @@ class AwlParser(object):
 		def finishCurToken(self):
 			self.curToken = self.curToken.strip()
 			if self.curToken:
-				self.tokens.append(self.curToken)
+				self.addToken(self.curToken)
 			self.curToken = ""
 
 		def finishStatement(self):
@@ -347,8 +360,10 @@ class AwlParser(object):
 		return self.__inAnyHeader() or\
 		       self.state == self.STATE_GLOBAL
 
-	def __tokenize(self, data):
+	def __tokenize(self, data, sourceId, sourceName):
 		self.reset()
+		self.tree.sourceId = sourceId
+		self.tree.sourceName = sourceName
 		self.lineNr = 1
 
 		t = self.TokenizerState(self)
@@ -601,6 +616,7 @@ class AwlParser(object):
 	def __parseInstruction(self, t):
 		insn = RawAwlInsn(self.tree.curBlock)
 		insn.setLineNr(t.tokensLineNr)
+		insn.setSourceId(self.tree.sourceId)
 		if t.tokens[0].endswith(":"):
 			# First token is a label
 			if len(t.tokens) <= 1:
@@ -966,16 +982,40 @@ class AwlParser(object):
 		insn = self.__parseInstruction(t)
 		self.tree.curBlock.insns.append(insn)
 
-	def parseFile(self, filename):
-		self.parseData(awlFileRead(filename))
+	def parseSource(self, awlSource):
+		"""Parse an AWL source.
+		awlSource is an AwlSource instance."""
+		self.parseData(awlSource.sourceBytes,
+			       sourceId = awlSource.identNr,
+			       sourceName = awlSource.name)
 
-	def parseData(self, data):
-		self.flatLayout = not re.match(r'.*^\s*ORGANIZATION_BLOCK\s+.*',
-					       data, re.DOTALL | re.MULTILINE)
+	@classmethod
+	def sourceIsFlat(cls, sourceText):
+		"""Returns whether the source is 'flat'.
+		A flat source is one without block definitions and
+		just plain AWL code."""
+		haveDB = re.match(r'.*^\s*DATA_BLOCK\s+.*', sourceText,
+				  re.DOTALL | re.MULTILINE)
+		haveFB = re.match(r'.*^\s*FUNCTION_BLOCK\s+.*', sourceText,
+				  re.DOTALL | re.MULTILINE)
+		haveFC = re.match(r'.*^\s*FUNCTION\s+.*', sourceText,
+				  re.DOTALL | re.MULTILINE)
+		haveOB = re.match(r'.*^\s*ORGANIZATION_BLOCK\s+.*', sourceText,
+				  re.DOTALL | re.MULTILINE)
+		return not haveDB and not haveFB and not haveFC and not haveOB
+
+	def parseData(self, dataBytes, sourceId=None, sourceName=None):
 		try:
-			self.__tokenize(data)
+			data = dataBytes.decode(self.TEXT_ENCODING)
+		except UnicodeError as e:
+			raise AwlParserError("Could not decode AWL/STL charset.")
+		self.flatLayout = self.sourceIsFlat(data)
+		try:
+			self.__tokenize(data, sourceId, sourceName)
 		except AwlParserError as e:
 			e.setLineNr(self.lineNr)
+			e.setSourceId(sourceId)
+			e.setSourceName(sourceName)
 			raise e
 
 	def getParseTree(self):
