@@ -2,7 +2,7 @@
 #
 # AWL simulator - PLC core server client
 #
-# Copyright 2013 Michael Buesch <m@bues.ch>
+# Copyright 2013-2014 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -151,13 +151,15 @@ class AwlSimClient(object):
 		"""Shutdown all sockets and spawned processes."""
 
 		if self.serverProcess:
-			try:
-				msg = AwlSimMessage_SHUTDOWN()
-				status = self.__sendAndWaitFor_REPLY(msg)
-				if status != AwlSimMessage_REPLY.STAT_OK:
-					printError("AwlSimClient: Failed to shut down server via message")
-			except (AwlSimError, MaintenanceRequest) as e:
-				pass
+			if self.transceiver:
+				try:
+					msg = AwlSimMessage_SHUTDOWN()
+					status = self.__sendAndWaitFor_REPLY(msg)
+					if status != AwlSimMessage_REPLY.STAT_OK:
+						printError("AwlSimClient: Failed to shut "
+							"down server via message")
+				except (AwlSimError, MaintenanceRequest) as e:
+					pass
 
 			self.serverProcess.terminate()
 			self.serverProcess.wait()
@@ -299,17 +301,17 @@ class AwlSimClient(object):
 			raise AwlSimError("AwlSimClient: Failed to set run state")
 		return True
 
-	def loadCode(self, code):
+	def loadCode(self, codeSource):
 		if not self.transceiver:
 			return False
-		msg = AwlSimMessage_LOAD_CODE(code)
+		msg = AwlSimMessage_LOAD_CODE(codeSource)
 		status = self.__sendAndWaitFor_REPLY(msg, 10.0)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to load code")
 		return True
 
-	def loadSymbolTable(self, symTabBytes):
-		msg = AwlSimMessage_LOAD_SYMTAB(symTabBytes)
+	def loadSymbolTable(self, symTabSource):
+		msg = AwlSimMessage_LOAD_SYMTAB(symTabSource)
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to load symbol table")
@@ -355,11 +357,27 @@ class AwlSimClient(object):
 	# Set instruction state dumping.
 	# fromLine, toLine is the range of AWL line numbers for which
 	# dumping is enabled.
-	# If fromLine=0, dumping is disabled.
-	def setInsnStateDump(self, fromLine=1, toLine=0x7FFFFFFF, sync=True):
-		return self.__setOption("insn_state_dump",
-					"%d-%d" % (fromLine, toLine),
-					sync = sync)
+	def setInsnStateDump(self, enable=True,
+			     sourceId=0, fromLine=1, toLine=0x7FFFFFFF,
+			     sync=True):
+		if not self.transceiver:
+			return None
+		msg = AwlSimMessage_INSNSTATE_CONFIG(
+			flags = 0,
+			sourceId = sourceId,
+			fromLine = fromLine,
+			toLine = toLine)
+		if enable:
+			msg.flags |= msg.FLG_CLEAR
+		else:
+			msg.flags |= msg.FLG_CLEAR_ONLY
+		if sync:
+			msg.flags |= msg.FLG_SYNC
+			status = self.__sendAndWaitFor_REPLY(msg)
+			if status != AwlSimMessage_REPLY.STAT_OK:
+				raise AwlSimError("AwlSimClient: Failed to set insn state dump")
+		else:
+			self.transceiver.send(msg)
 
 	def getCpuSpecs(self):
 		if not self.transceiver:
@@ -383,7 +401,7 @@ class AwlSimClient(object):
 	# memAreas is a list of MemoryArea instances.
 	# The repetitionFactor tells whether to
 	#  - only run the request once (repetitionFactor=0)
-	#  - repeat on n'th every cycle (repetitionFactor=n)
+	#  - repeat on every n'th cycle (repetitionFactor=n)
 	# If sync is true, wait for a reply from the server.
 	def setMemoryReadRequests(self, memAreas, repetitionFactor=0, sync=False):
 		if not self.transceiver:
