@@ -2,7 +2,7 @@
 #
 # AWL simulator - PLC core server messages
 #
-# Copyright 2013 Michael Buesch <m@bues.ch>
+# Copyright 2013-2014 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ from awlsim.coreserver.memarea import *
 from awlsim.core.util import *
 from awlsim.core.datatypehelpers import *
 from awlsim.core.cpuspecs import *
+from awlsim.core.project import *
 
 import struct
 import socket
@@ -45,7 +46,7 @@ class AwlSimMessage(object):
 	#	Payload (optional)
 	hdrStruct = struct.Struct(str(">HHHHI"))
 
-	HDR_MAGIC		= 0x5710
+	HDR_MAGIC		= 0x5711
 	HDR_LENGTH		= hdrStruct.size
 
 	EnumGen.start
@@ -67,29 +68,49 @@ class AwlSimMessage(object):
 	MSG_ID_REQ_MEMORY	= EnumGen.item
 	MSG_ID_MEMORY		= EnumGen.item
 	MSG_ID_INSNSTATE	= EnumGen.item
+	MSG_ID_INSNSTATE_CONFIG	= EnumGen.item
 	EnumGen.end
 
-	_strLenStruct = struct.Struct(str(">H"))
+	_bytesLenStruct = struct.Struct(str(">I"))
 
 	@classmethod
 	def packString(cls, string):
 		try:
-			data = string.encode("utf-8", "strict")
-			return cls._strLenStruct.pack(len(data)) + data
-		except (UnicodeError, struct.error) as e:
+			if not string:
+				string = ""
+			return cls.packBytes(string.encode("utf-8", "strict"))
+		except UnicodeError as e:
+			raise ValueError
+
+	@classmethod
+	def packBytes(cls, _bytes):
+		try:
+			if not _bytes:
+				_bytes = b""
+			if len(_bytes) > 0xFFFFFFFF:
+				raise ValueError
+			return cls._bytesLenStruct.pack(len(_bytes)) + _bytes
+		except struct.error as e:
 			raise ValueError
 
 	@classmethod
 	def unpackString(cls, data, offset = 0):
 		try:
-			(length, ) = cls._strLenStruct.unpack_from(data, offset)
-			strBytes = data[offset + cls._strLenStruct.size :
-					offset + cls._strLenStruct.size + length]
-			if len(strBytes) != length:
+			_bytes, count = cls.unpackBytes(data, offset)
+			return (_bytes.decode("utf-8", "strict"), count)
+		except UnicodeError as e:
+			raise ValueError
+
+	@classmethod
+	def unpackBytes(cls, data, offset = 0):
+		try:
+			(length, ) = cls._bytesLenStruct.unpack_from(data, offset)
+			_bytes = data[offset + cls._bytesLenStruct.size :
+				      offset + cls._bytesLenStruct.size + length]
+			if len(_bytes) != length:
 				raise ValueError
-			return (strBytes.decode("utf-8", "strict"),
-				cls._strLenStruct.size + length)
-		except (UnicodeError, struct.error) as e:
+			return (_bytes, cls._bytesLenStruct.size + length)
+		except struct.error as e:
 			raise ValueError
 
 	def __init__(self, msgId, seq=0):
@@ -200,37 +221,52 @@ class AwlSimMessage_EXCEPTION(AwlSimMessage):
 			raise TransferError("EXCEPTION: Unicode error")
 		return cls(text)
 
-class AwlSimMessage_LOAD_SYMTAB(AwlSimMessage):
-	def __init__(self, symTabBytes):
-		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_LOAD_SYMTAB)
-		self.symTabBytes = symTabBytes
+class _AwlSimMessage_source(AwlSimMessage):
+	sourceClass = None
 
-	def toBytes(self):
-		return AwlSimMessage.toBytes(self, len(self.symTabBytes)) + self.symTabBytes
+	# Payload header struct:
+	#	Source identifier (32 bit)
+	plStruct = struct.Struct(str(">I"))
 
-	@classmethod
-	def fromBytes(cls, payload):
-		return cls(payload)
-
-class AwlSimMessage_LOAD_CODE(AwlSimMessage):
-	def __init__(self, code):
-		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_LOAD_CODE)
-		self.code = code
+	def __init__(self, msgId, source):
+		AwlSimMessage.__init__(self, msgId)
+		self.source = source
 
 	def toBytes(self):
 		try:
-			code = self.code.encode()
-			return AwlSimMessage.toBytes(self, len(code)) + code
-		except UnicodeError:
-			raise TransferError("LOAD_CODE: Unicode error")
+			pl = self.plStruct.pack(self.source.identNr) +\
+				self.packString(self.source.name) +\
+				self.packString(self.source.filepath) +\
+				self.packBytes(self.source.sourceBytes)
+			return AwlSimMessage.toBytes(self, len(pl)) + pl
+		except ValueError:
+			raise TransferError("SOURCE: Data format error")
 
 	@classmethod
 	def fromBytes(cls, payload):
 		try:
-			code = payload.decode()
-		except UnicodeError:
-			raise TransferError("LOAD_CODE: Unicode error")
-		return cls(code)
+			(identNr, ) = cls.plStruct.unpack_from(payload, 0)
+			count = cls.plStruct.size
+			name, cnt = cls.unpackString(payload, count)
+			count += cnt
+			filepath, cnt = cls.unpackString(payload, count)
+			count += cnt
+			sourceBytes, cnt = cls.unpackBytes(payload, count)
+		except (ValueError, struct.error) as e:
+			raise TransferError("SOURCE: Data format error")
+		return cls(cls.sourceClass(identNr, name, filepath, sourceBytes))
+
+class AwlSimMessage_LOAD_SYMTAB(_AwlSimMessage_source):
+	sourceClass = SymTabSource
+
+	def __init__(self, source):
+		_AwlSimMessage_source.__init__(self, AwlSimMessage.MSG_ID_LOAD_SYMTAB, source)
+
+class AwlSimMessage_LOAD_CODE(_AwlSimMessage_source):
+	sourceClass = AwlSource
+
+	def __init__(self, source):
+		_AwlSimMessage_source.__init__(self, AwlSimMessage.MSG_ID_LOAD_CODE, source)
 
 class AwlSimMessage_LOAD_HW(AwlSimMessage):
 	def __init__(self, name, paramDict):
@@ -509,45 +545,88 @@ class AwlSimMessage_MEMORY(AwlSimMessage):
 
 class AwlSimMessage_INSNSTATE(AwlSimMessage):
 	# Payload data struct:
+	#	AWL source ident number (32 bit)
 	#	AWL line number (32 bit)
 	#	Serial number. Reset to 0 on cycle exit. (32 bit)
-	#	Flags (16 bit)
+	#	Flags (16 bit) (currently unused. Set to 0)
 	#	CPU status word (16 bit)
 	#	CPU ACCU 1 (32 bit)
 	#	CPU ACCU 2 (32 bit)
+	#	CPU ACCU 3 (32 bit)
+	#	CPU ACCU 4 (32 bit)
 	#	CPU AR 1 (32 bit)
 	#	CPU AR 2 (32 bit)
 	#	CPU DB register (16 bit)
 	#	CPU DI register (16 bit)
-	plDataStruct = struct.Struct(str(">IIHHIIIIHH"))
+	plDataStruct = struct.Struct(str(">IIIHHIIIIIIHH"))
 
-	def __init__(self, lineNr, serial, flags, stw, accu1, accu2, ar1, ar2, db, di):
+	def __init__(self, sourceId, lineNr, serial, flags, stw, accu1, accu2, accu3, accu4, ar1, ar2, db, di):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_INSNSTATE)
+		self.sourceId = sourceId
 		self.lineNr = lineNr
 		self.serial = serial
 		self.flags = flags
 		self.stw = stw
 		self.accu1 = accu1
 		self.accu2 = accu2
+		self.accu3 = accu3
+		self.accu4 = accu4
 		self.ar1 = ar1
 		self.ar2 = ar2
 		self.db = db
 		self.di = di
 
 	def toBytes(self):
-		pl = self.plDataStruct.pack(self.lineNr, self.serial,
+		pl = self.plDataStruct.pack(
+			self.sourceId, self.lineNr, self.serial,
 			self.flags, self.stw, self.accu1, self.accu2,
-			self.ar1, self.ar2, self.db, self.di)
+			self.accu3, self.accu4, self.ar1, self.ar2,
+			self.db, self.di)
 		return AwlSimMessage.toBytes(self, len(pl)) + pl
 
 	@classmethod
 	def fromBytes(cls, payload):
 		try:
-			lineNr, serial, flags, stw, accu1, accu2, ar1, ar2, db, di =\
+			sourceId, lineNr, serial, flags, stw, accu1, accu2, accu3, accu4, ar1, ar2, db, di =\
 				cls.plDataStruct.unpack_from(payload, 0)
 		except (struct.error, IndexError) as e:
 			raise TransferError("INSNSTATE: Invalid data format")
-		return cls(lineNr, serial, flags, stw, accu1, accu2, ar1, ar2, db, di)
+		return cls(sourceId, lineNr, serial, flags, stw, accu1, accu2, accu3, accu4, ar1, ar2, db, di)
+
+class AwlSimMessage_INSNSTATE_CONFIG(AwlSimMessage):
+	# Payload data struct:
+	#	Flags (32 bit)
+	#	AWL source ident number (32 bit)
+	#	From AWL line (32 bit)
+	#	To AWL line (32 bit)
+	plDataStruct = struct.Struct(str(">IIII"))
+
+	# Flags:
+	FLG_SYNC		= 1 << 0 # Synchronous status reply.
+	FLG_CLEAR_ONLY		= 1 << 1 # Just clear current settings.
+	FLG_CLEAR		= 1 << 2 # Clear, then apply settings.
+
+	def __init__(self, flags, sourceId, fromLine, toLine):
+		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_INSNSTATE_CONFIG)
+		self.flags = flags
+		self.sourceId = sourceId
+		self.fromLine = fromLine
+		self.toLine = toLine
+
+	def toBytes(self):
+		pl = self.plDataStruct.pack(
+			self.flags, self.sourceId,
+			self.fromLine, self.toLine)
+		return AwlSimMessage.toBytes(self, len(pl)) + pl
+
+	@classmethod
+	def fromBytes(cls, payload):
+		try:
+			flags, sourceId, fromLine, toLine =\
+				cls.plDataStruct.unpack_from(payload, 0)
+		except (struct.error, IndexError) as e:
+			raise TransferError("INSNSTATE_CONFIG: Invalid data format")
+		return cls(flags, sourceId, fromLine, toLine)
 
 class AwlSimMessageTransceiver(object):
 	class RemoteEndDied(Exception): pass
@@ -571,6 +650,7 @@ class AwlSimMessageTransceiver(object):
 		AwlSimMessage.MSG_ID_REQ_MEMORY		: AwlSimMessage_REQ_MEMORY,
 		AwlSimMessage.MSG_ID_MEMORY		: AwlSimMessage_MEMORY,
 		AwlSimMessage.MSG_ID_INSNSTATE		: AwlSimMessage_INSNSTATE,
+		AwlSimMessage.MSG_ID_INSNSTATE_CONFIG	: AwlSimMessage_INSNSTATE_CONFIG,
 	}
 
 	def __init__(self, sock, peerInfoString):
