@@ -28,7 +28,12 @@ from awlsim.gui.awlsimclient import *
 
 
 class CpuWidget(QWidget):
+	# Signal: The CPU run-state changed
 	runStateChanged = Signal(int)
+	# Signal: The online-diag state changed
+	onlineDiagChanged = Signal(bool)
+	# Signal: Have a new instruction dump
+	haveInsnDump = Signal(AwlSimMessage_INSNSTATE)
 
 	EnumGen.start
 	STATE_STOP	= EnumGen.item
@@ -46,11 +51,9 @@ class CpuWidget(QWidget):
 
 		client = self.mainWidget.getSimClient()
 		client.haveCpuDump.connect(self.__handleCpuDump)
-		client.haveInsnDump.connect(self.mainWidget.codeEdit.updateCpuStats_afterInsn)
+		client.haveInsnDump.connect(self.haveInsnDump)
 		client.haveMemoryUpdate.connect(self.__handleMemoryUpdate)
 
-		self.mainWidget.codeEdit.visibleRangeChanged.connect(self.__updateOnlineViewState)
-
 		group = QGroupBox("CPU status", self)
 		group.setLayout(QGridLayout(group))
 		self.runButton = QRadioButton("RUN", group)
@@ -172,9 +175,10 @@ class CpuWidget(QWidget):
 		self.runButton.setEnabled(False) # Redraws the radio button
 		self.runButton.setEnabled(True)
 
-		ob1_awl = self.mainWidget.getCodeEditWidget().getCode()
-		if not ob1_awl.strip():
-			MessageBox.error(self, "No AWL/STL code available. Cannot run.")
+		project = self.mainWidget.getProject()
+		awlSources = self.mainWidget.projectWidget.getAwlSources()
+		symTabSources = self.mainWidget.projectWidget.getSymTabSources()
+		if not all(awlSources) or not all(symTabSources):
 			self.stop()
 			return
 
@@ -211,13 +215,19 @@ class CpuWidget(QWidget):
 			client.setRunState(False)
 			client.reset()
 
-			self.mainWidget.cpuConfigDialog.uploadToCPU()
+			client.setCpuSpecs(project.getCpuSpecs())
+			client.enableOBTempPresets(project.getObTempPresetsEn())
+			client.enableExtendedInsns(project.getExtInsnsEn())
+
 			self.__uploadMemReadAreas()
 			self.__updateOnlineViewState()
 
 			self.__setState(self.STATE_LOAD)
 			client.loadHardwareModule("dummy")
-			client.loadCode(ob1_awl)
+			for symTabSource in symTabSources:
+				client.loadSymbolTable(symTabSource)
+			for awlSource in awlSources:
+				client.loadCode(awlSource)
 			client.setRunState(True)
 		except AwlParserError as e:
 			MessageBox.handleAwlParserError(self, e)
@@ -298,15 +308,20 @@ class CpuWidget(QWidget):
 				self.stop()
 
 	def __updateOnlineViewState(self):
-		en = self.onlineViewCheckBox.checkState() == Qt.Checked
-		self.mainWidget.codeEdit.enableCpuStats(en)
+		onlineDiagEn = self.onlineViewCheckBox.checkState() == Qt.Checked
+		self.onlineDiagChanged.emit(onlineDiagEn)
+
+	def updateVisibleLineRange(self, source, fromLine, toLine):
+		onlineDiagEn = self.onlineViewCheckBox.checkState() == Qt.Checked
 		try:
 			client = self.mainWidget.getSimClient()
-			if en:
-				fromLine, toLine = self.mainWidget.codeEdit.getVisibleLineRange()
-				client.setInsnStateDump(fromLine, toLine, sync=False)
+			if onlineDiagEn and source:
+				client.setInsnStateDump(enable=True,
+							sourceId=source.identNr,
+							fromLine=fromLine, toLine=toLine,
+							sync=False)
 			else:
-				client.setInsnStateDump(0, 0, sync=False)
+				client.setInsnStateDump(enable=False, sync=False)
 		except AwlSimError as e:
 			MessageBox.handleAwlSimError(self,
 				"Failed to setup instruction dumping", e)
